DROP TABLE spam_tracker;
CREATE TABLE spam_tracker (
  sid serial PRIMARY KEY,
  source varchar(64) NOT NULL default '', -- node, comment, trackback, etc...
  id integer NOT NULL default 0,
  probability smallint default 0,
  hostname varchar(128) NOT NULL default '', -- source IP
  hash char(32) NOT NULL default '',
  timestamp integer default 0
);
DROP INDEX spam_tracker_id_key;
DROP INDEX spam_tracker_probability_key;
DROP INDEX spam_tracker_source_key;
DROP INDEX spam_tracker_hostname_key;
DROP INDEX spam_tracker_hash_key;
CREATE INDEX spam_tracker_id_key ON spam_tracker (id);
CREATE INDEX spam_tracker_probability_key ON spam_tracker (probability);
CREATE INDEX spam_tracker_source_key ON spam_tracker (source);
CREATE INDEX spam_tracker_hostname_key ON spam_tracker (hostname);
CREATE INDEX spam_tracker_hash_key ON spam_tracker (hash);

DROP TABLE spam_tokens;
CREATE TABLE spam_tokens (
  tid serial PRIMARY KEY,
  token varchar(255) NOT NULL default '',
  spam integer DEFAULT 0
       	       CONSTRAINT spam_tokens_spam_unsigned
	       CHECK (spam >= 0),
  notspam integer DEFAULT 0
  	  	  CONSTRAINT spam_tokens_notspam_unsigned
		  CHECK (notspam >= 0),
  probability integer DEFAULT 1
  	      	      CONSTRAINT spam_tokens_probability_positive
		      CHECK (probability > 0),
  last integer DEFAULT 0
);
DROP INDEX spam_tokens_token_key;
DROP INDEX spam_tokens_spam_key;
DROP INDEX spam_tokens_notspam_key;
DROP INDEX spam_tokens_probability_key;
DROP INDEX spam_tokens_last_key;
CREATE UNIQUE INDEX spam_tokens_token_key ON spam_tokens (token);
CREATE INDEX spam_tokens_spam_key ON spam_tokens (spam);
CREATE INDEX spam_tokens_notspam_key ON spam_tokens (notspam);
CREATE INDEX spam_tokens_probability_key ON spam_tokens (probability);
CREATE INDEX spam_tokens_last_key ON spam_tokens (last);


DROP TABLE spam_custom;
CREATE TABLE spam_custom (
  scid serial PRIMARY KEY,
  filter varchar(255) NOT NULL default '',
  style smallint default 0, -- text, regex, URL
  effect smallint default 0, 
  action smallint default 0,
  matches integer default 0,
  last integer default 0
);
DROP INDEX spam_custom_filter_key;
DROP INDEX spam_custom_matches_key;
DROP INDEX spam_custom_last_key;
CREATE INDEX spam_custom_filter_key ON spam_custom (filter);
CREATE INDEX spam_custom_matches_key ON spam_custom (matches);
CREATE INDEX spam_custom_last_key ON spam_custom (last);

INSERT INTO spam_custom VALUES (1, '/(adipex|cialis|phentermine|viagra)/i', 1, 1, 2, 0, 0);
INSERT INTO spam_custom VALUES (2, '/casino games|poker online|texas hold\\''em|texas holdem/i', 1, 0, 2, 0, 0);
INSERT INTO spam_custom VALUES (3, '/(american|casino|free|grand|online|party)( |-)?(blackjack|casino|gambling|poker)/i', 1, 0, 2, 0, 0);
INSERT INTO spam_custom VALUES (4, 'diet pill', 0, 0, 2, 0, 0);

DROP TABLE spam_reported;
CREATE TABLE spam_reported (
  rid serial PRIMARY KEY,
  source varchar(64) NOT NULL default '', -- node, comment, trackback, etc...
  id integer NOT NULL default 0,
  uid integer NOT NULL default 0,
  hostname varchar(128) NOT NULL default '', -- source IP
  feedback text NULL default '',
  timestamp integer default 0
);

DROP TABLE spam_log;
CREATE TABLE spam_log (
  sid serial PRIMARY KEY,
  source varchar(64) NOT NULL default '', -- node, comment, trackback, etc...
  id integer NOT NULL default 0,
  uid integer NOT NULL default 0,
  hostname varchar(128) NOT NULL default '', -- source IP
  entry varchar(255) NOT NULL default '',
  timestamp integer default 0
);

DROP INDEX spam_log_source_key;
DROP INDEX spam_log_id_key;
DROP INDEX spam_log_timestamp_key;

CREATE INDEX spam_log_source_key ON spam_log (source);
CREATE INDEX spam_log_id_key ON spam_log (id);
CREATE INDEX spam_log_timestamp_key ON spam_log (timestamp);
