<?php

/**
 * Code borrowed heavily from Drupal's update.php and update.inc. 
 */

$access_check = TRUE;

if (!ini_get('safe_mode')) {
  set_time_limit(180);
}

include_once 'includes/bootstrap.inc';
include_once 'includes/common.inc';

$spam_updates = array(
  '2.0.11 -> 2.0.12' => 'update_01',
  '2.0.12 -> 2.0.13' => 'update_02',
);

function update_query($sql) {
  $result = db_query($sql);
  if ($result) {
    return array('1', nl2br(htmlentities($sql)) ." ", "<div style=\"color: green;\">OK</div>\n");
  }
  else {
    return array('0', nl2br(htmlentities($sql)) ." ", "<div style=\"color: red;\">FAILED</div>\n");
  }
}

if (($access_check == 0) || ($user->uid == 1)) {
  update_page();
}
else {
  print update_page_header("Access denied.");
      print "<p>Access denied.  You are not authorized to access this page.  Please log in as the admin user (the first user you created). If you cannot log in, you will have to edit <code>spam_update.php</code> to bypass this access check.  To do this:</p>";
    print "<ol>";
    print " <li>With a text editor find the spam_update.php file on your system. It should be in the main Drupal directory that you installed all the files into.</li>";
    print " <li>There is a line near top of spam_update.php that says <code>\$access_check = TRUE;</code>. Change it to <code>\$access_check = FALSE;</code>.</li>";
    print " <li>As soon as the script is done, you must change the spam_update.php script back to its original form to <code>\$access_check = TRUE;</code>.</li>";
    print "</ol>";

  print update_page_footer();
}

function update_page_header($title) {
  $output = "<html><head><title>$title</title>";
  $output .= <<<EOF
      <link rel="stylesheet" type="text/css" media="print" href="misc/print.css" />
      <style type="text/css" title="layout" media="Screen">
        @import url("misc/drupal.css");
      </style>
EOF;
  $output .= "</head><body>";
  $output .= "<div id=\"logo\"><a href=\"http://drupal.org/\"><img src=\"misc/druplicon-small.png\" alt=\"Druplicon - Drupal logo\" title=\"Druplicon - Drupal logo\" /></a></div>";
  $output .= "<div id=\"update\"><h1>$title</h1>";
  return $output;
}

function update_page_footer() {
  return "</div></body></html>";
}

function update_page() {
  global $user, $spam_updates;

  if (isset($_POST['edit'])) {
    $edit = $_POST['edit'];
  }
  if (isset($_POST['op'])) {
    $op = $_POST['op'];
  }

  switch ($op) {
    case 'Update':
      print update_page_header('Update Drupal spam database');
      $links[] = "<a href=\"index.php\">main page</a>";
      $links[] = "<a href=\"index.php?q=admin\">administration pages</a>";
      $links[] = "<a href=\"index.php?q=admin/spam\">spam administration pages</a>";
      print theme("item_list", $links);
      if ($edit["start"] == -1) {
        print "No updates to perform.";
      }
      else {
        update_data($edit["start"]);
      }
      print "<br />Updates were attempted. If you see no failures above, you may proceed happily to the <a href=\"index.php?q=admin/spam\">spam administration pages</a>.";
      print " Otherwise, you may need to update your database manually.";
      print update_page_footer();
      break;
    default:
      $start = variable_get('spam_update_start', 0);
      $versions[] = 'All';
      $i = 1;
      foreach ($spam_updates as $version => $sql) {
        $versions[$i++] = $version;
	if ($version == $start) {
	  $selected = $i;
	}
      }
      $versions[$i] = 'No more updates available';

      $form = form_select('Perform updates', 'start', (isset($selected) ? $selected : -1), $versions, 'This will default to the first available update since the last spam database update you\'ve performed.');
      print update_page_header('Update Drupal spam database');
      $form .= form_submit('Update');
      print update_page_footer();
      print form($form);
  }
}

function update_data($start) {
  global $spam_updates;
  $spam_updates = array_slice($spam_updates, ($start-- ? $start : 0));
  foreach ($spam_updates as $version => $func) {
    print "<strong>$version</strong><br />\n<pre>\n";
    $ret = $func();
    foreach ($ret as $return) {
      print $return[1];
      print $return[2];
    }
    variable_set('spam_update_start', $version);
    print "</pre>\n";
  }
  db_query('DELETE FROM {cache}');
}



function update_01() {
  $ret = array();


  if ($GLOBALS['db_type'] == 'mysql') {
    $ret[] = update_query("ALTER TABLE {spam_tokens} RENAME TO {spam_tokens_old}");
    $ret[] = update_query("CREATE TABLE {spam_tokens} (
      tid int(10) unsigned NOT NULL auto_increment,
      token varchar(255) NOT NULL default '',
      spam int(10) unsigned default '0',
      notspam int(10) unsigned default '0',
      probability int(10) unsigned default '0',
      last int(11) unsigned default '0',
      PRIMARY KEY tid (tid),
      UNIQUE KEY token (token),
      KEY spam (spam),
      KEY notspam (notspam),
      KEY probability (probability),
      KEY last (last)
    ) TYPE=MyISAM;");
  }
  else {
    $ret[] = update_query("ALTER TABLE spam_tokens RENAME TO spam_tokens_old");
    $ret[] = update_query("ALTER TABLE spam_tokens_tid_seq RENAME TO spam_tokens_old_tid_seq");
    $ret[] = update_query("ALTER TABLE spam_tokens_pkey RENAME TO spam_tokens_old_pkey");
    $ret[] = update_query("ALTER TABLE spam_tokens_token_key RENAME TO spam_tokens_old_token_key");
    $ret[] = update_query("ALTER TABLE spam_tokens_spam_key RENAME TO spam_tokens_old_spam_key");
    $ret[] = update_query("ALTER TABLE spam_tokens_notspam_key RENAME TO spam_tokens_old_notspam_key");
    $ret[] = update_query("ALTER TABLE spam_tokens_probability_key RENAME TO spam_tokens_old_probability_key");
    $ret[] = update_query("ALTER TABLE spam_tokens_last_key RENAME TO spam_tokens_old_last_key");
    $ret[] = update_query("CREATE TABLE spam_tokens (
      tid serial PRIMARY KEY,
      token varchar(255) UNIQUE NOT NULL DEFAULT '',
      spam integer default 0,
      notspam integer default 0,
      probability integer default 0,
      last integer default 0 )");
    $ret[] = update_query("CREATE INDEX spam_tokens_spam_key ON spam_tokens (spam)");
    $ret[] = update_query("CREATE INDEX spam_tokens_notspam_key ON spam_tokens (notspam)");
    $ret[] = update_query("CREATE INDEX spam_tokens_probability_key ON spam_tokens (probability)");
    $ret[] = update_query("CREATE INDEX spam_tokens_last_key ON spam_tokens (last)");
  }

  $orig = $new = $url = 0;

  $result = db_query('SELECT * FROM spam_tokens_old');
  while ($old = db_fetch_object($result)) {
    $orig++;

    // test if URI, if so, just save it
    if (substr($old->token, 0, 4) == 'URL*') {
      $url++;
      $new += update_01_save_token($old->token, $old);
    }
    else {
      $token = preg_replace('/^header\*/', '', $old->token, 1);
      if ($token != $old->token) {
        $header = 'header*';
      }
      else {
        $header = '';
      }
  
      $tokens = array();
      $sanitized = preg_replace('(http://|https://|ftp://|mailto:)', '', $token);
      $sanitized = preg_replace("'(www\.)|(</a>)|(href=)|(target=)|(src=)'i", '', $sanitized);
      $sanitized = preg_replace("/[()\{\}\[\]#.,]/", '', $sanitized);
      $sanitized = strtolower($sanitized);
      $delimiters = " \t\n\r-_<>'\"`/|*%^&+=~:;?";
      $tok = strtok($sanitized, $delimiters);
      while ($tok !== FALSE) {
        // if longer than 255 characters, truncate the token
        $tokens[] = htmlspecialchars(substr("$header$tok", 0, 254));
        $tok = strtok($delimiters);
      }
      foreach ($tokens as $t) {
        if ($t != '') {
          $new += update_01_save_token($t, $old);
        }
      }
    }
  }

  if ($GLOBALS['db_type'] == 'mysql') {
    $ret[] = update_query('DROP TABLE {spam_tokens_old}');
  }
  else {
    $ret[] = update_query('DROP TABLE spam_tokens_old');
  }
  
  print "<p>Converting $orig tokens into $new tokens.</p>";
  return $ret;
}

function update_01_save_token($token, $old) {
  $existing = db_fetch_object(db_query("SELECT * from spam_tokens WHERE token = '%s'", $token));
  if (isset($existing->token)) {
    $total = $existing->spam + $existing->notspam + $old->spam + $old->notspam;
    $probability = (($existing->spam + $old->spam) / $total * 100);
    if ($probability > 99) {
      $probability = 99;
    }
    else if ($probability < 1) {
      $probability = 1;
    }
    db_query("UPDATE spam_tokens SET spam = %d, notspam = %d, probability = %d, last = %d WHERE token = '%s'", $existing->spam + $old->spam, $existing->notspam + $old->notspam, $probability, ($existing->last >= $old->last ? $existing->last : $old->last), $existing->token);
    return 0;
  }
  else {
    // we shouldn't have to make changes here, but let's be sure things are sane
    $spam = $old->spam < 1 ? 1 : $old->spam;
    $notspam = $old->notspam < 1 ? 1 : $old->notspam;
    $total = $spam + $notspam;
    $probability = ($spam / $total * 100);
    if ($probability > 99) {
      $probability = 99;
    }
    else if ($probability < 1) {
      $probability = 1;
    }
    db_query("INSERT INTO spam_tokens (token, spam, notspam, probability, last) VALUES('%s', %d, %d, %d, %d)", $token, $spam, $notspam, $probability, $old->last);
    return 1;
  }
}

function update_02() {
  $ret = array();

  if ($GLOBALS['db_type'] == 'mysql') {
    $ret[] = update_query("CREATE TABLE spam_reported (
      rid int(11) unsigned NOT NULL auto_increment,
      source varchar(64) NOT NULL default '',
      id int(11) unsigned NOT NULL default '0',
      uid int(10) unsigned NOT NULL default '0',
      hostname varchar(128) NOT NULL default '',
      feedback text NULL default '',
      timestamp int(11) unsigned default '0',
      PRIMARY KEY rid (rid),
      KEY source (source),
      KEY id (id),
      KEY uid (uid)
    ) TYPE=MyISAM;");
  }
  else {
    $ret[] = update_query("CREATE TABLE spam_reported (
      rid serial PRIMARY KEY,
      source varchar(64) NOT NULL default '',
      id integer NOT NULL default 0,
      uid integer NOT NULL default 0,
      hostname varchar(128) NOT NULL default '',
      feedback text NULL default '',
      timestamp integer default 0);");
  }

  return $ret;
}

?>
